/*
 * CollectionTransectGisVisualisation.java
 *
 * Created on 13. August 2003, 16:40
 */

package net.metadiversity.diversity.navigator.gui;

//import net.metadiversity.diversity.navigator.checklists.database.DBChecklist;

import java.sql.*;            // JDBC

import net.metadiversity.diversity.navigator.util.CollectionObservatory;
import net.metadiversity.diversity.navigator.connection.ConnectionInfo;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

import javax.xml.rpc.ParameterMode;

/**
 *
 * @author  Dieter Neubacher
 */
public class CollectionTransectGisVisualisation extends javax.swing.JPanel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3257569524839887158L;

    static Logger logger = Logger.getLogger( CollectionTransectGisVisualisation.class.getName() ); 
       
    // Variables declaration
    
    final static java.lang.String bundleFullPath = "net/metadiversity/diversity/navigator/gui/CollectionTransectWebServiceBundle";
    static java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle( bundleFullPath );
    
    private final static String EMPTY_MAP_NAME          = bundle.getString( "EMPTY_MAP_NAME" );
    private final static String MAP_LOCATION            = bundle.getString( "MAP_LOCATION" );    
    private final static String MAP_EXTENSION           = bundle.getString( "MAP_EXTENSION" );    
    private final static String WEB_SERVICE_ENDPOINT    = bundle.getString( "WEB_SERVICE_ENDPOINT" );            
    // GUI Strings
    private final static String CHAR_NAME_LABEL         = bundle.getString( "LABEL_CHAR_NAME" );
    private final static String CHAR_STATE_LABEL        = bundle.getString( "LABEL_CHAR_STATE" );
    private final static String SHOW_IMAGE_LABEL        = bundle.getString( "LABEL_SHOW_IMAGE" );
    // Database table names
    private String TABLE_CHAR;            
    private String TABLE_CS;
    private String TABLE_DESCR; 
    private String TABLE_ITEM;
    // GUI elements
    private javax.swing.JPanel      mainPanel;
    private javax.swing.JPanel      jPanel;     // selection
    private javax.swing.JLabel      jLabelCharName;
    private javax.swing.JButton     jButtonShowImage;
    private javax.swing.JLabel      jLabelCharState;
    private javax.swing.JComboBox   jComboBoxCharName;
    private javax.swing.JComboBox   jComboBoxCharState;
    //----------------------------------------------
    private java.lang.String        charNameSelection;
    private java.lang.String        charStateSelection;
    
    private java.util.Vector        charNameModel;
    private java.util.Vector        charStateModel;
    private java.util.HashMap       charStateHashMap;
    private java.util.Vector        descriptionResults;
    
    private ImageViewer              imageViewer;
    
    private Connection conDescription;     // ubt-description DB-Connection
    private Connection conCollection;      // ubt-collection  DB-Connection
    
    private CollectionObservatory ct;
    
    private final static boolean ENABLE_CHAR_STATE_SELECTION = false;
    //--------------------------------------------------------------------------
    /** 
     * Creates new form ChecklistsGisVisualisation 
     */
    //--------------------------------------------------------------------------
    public CollectionTransectGisVisualisation( Connection connectionDescription, Connection connectionCollection, String descriptionTablePrefix )
    {
        String DATABASE_TABLE_PREFIX = descriptionTablePrefix;
        
        TABLE_CHAR              = " \"" + DATABASE_TABLE_PREFIX + "_CHAR" +"\" ";
        TABLE_CS                = " \"" + DATABASE_TABLE_PREFIX + "_CS" +"\" ";
        TABLE_DESCR             = " \"" + DATABASE_TABLE_PREFIX + "_DESCR" +"\" ";
        TABLE_ITEM              = " \"" + DATABASE_TABLE_PREFIX + "_ITEM" +"\" ";
        
        // read transect informations
        ct = new CollectionObservatory();
        
        charNameModel = new java.util.Vector();
        charStateModel = new java.util.Vector();
        charStateModel.add( "ALL" );
        charStateHashMap = new java.util.HashMap();
        
        conDescription = connectionDescription;
        conCollection  = connectionCollection;
        try
        {
            if( conDescription == null || conCollection == null )
            {
                logger.error( "Can't create connection " );
            }
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        charNameModel = setupCharNames();        
        initComponents();
    }
    //--------------------------------------------------------------------------
    /**
     * @throws Throwable
     */    
    //--------------------------------------------------------------------------
    protected void finalize() throws java.lang.Throwable
    {
        // close DataBaseConnection
        if( conDescription != null )
        {
            logger.info( "ChecklistGisVisualisation: close description database connection" );
            conDescription.close();
        }    
        if( conCollection != null )
        {
            logger.info( "ChecklistGisVisualisation: close collection database connection" );
            conCollection.close();
        }    
        super.finalize();        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void initComponents()
    {
        mainPanel           = new javax.swing.JPanel();
        jPanel              = new javax.swing.JPanel();
        jLabelCharName      = new javax.swing.JLabel();
        jComboBoxCharName   = new javax.swing.JComboBox();
        jLabelCharState     = new javax.swing.JLabel();
        if( ENABLE_CHAR_STATE_SELECTION )
        {    
            jComboBoxCharState  = new javax.swing.JComboBox();
        }
        jButtonShowImage    = new javax.swing.JButton();
        try
        {
            // load empty map
            imageViewer = new ImageViewer( new java.net.URL( MAP_LOCATION + EMPTY_MAP_NAME + MAP_EXTENSION) );
        }    
        catch( java.net.MalformedURLException ex )
        {
            logger.error( MAP_LOCATION + EMPTY_MAP_NAME );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //--------------------------
        // Character name selection
        //--------------------------
        jLabelCharName.setText( CHAR_NAME_LABEL );
        jPanel.add( jLabelCharName );
        jComboBoxCharName.setEditable( true );
        jComboBoxCharName.setModel( new javax.swing.DefaultComboBoxModel( charNameModel ) );
        // ??? Don't work: 
        jComboBoxCharName.setSize( new java.awt.Dimension( 25,200 ) );
// ???        jComboBoxCharName.getPrototypeDisplayValue().
        jComboBoxCharName.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                jComboBoxCharNameActionPerformed( evt );
            }
        });
        jPanel.add( jComboBoxCharName );
        //---------------------------
        // Character state selection
        //---------------------------
        if( ENABLE_CHAR_STATE_SELECTION )
        {
            jLabelCharState.setText( CHAR_STATE_LABEL );
            jPanel.add( jLabelCharState );
            jComboBoxCharState.setModel( new javax.swing.DefaultComboBoxModel( charStateModel ) );
            jComboBoxCharState.setEditable( true );
            jPanel.add( jComboBoxCharState );
        }
        //-------------------------------
        // get the image wia WEB-Service
        //-------------------------------
        jButtonShowImage.setText( SHOW_IMAGE_LABEL );
        jButtonShowImage.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                jButtonShowImageActionPerformed(evt);
            }
        });
        jPanel.add( jButtonShowImage );
        
        // mainPanel
        this.setLayout( new java.awt.BorderLayout() );
        this.add( jPanel, java.awt.BorderLayout.NORTH );
        this.add( imageViewer, java.awt.BorderLayout.CENTER );
    }
    
    private String getCharacterName()
    {
//        String index = ( (IndexSelection) jComboBoxCharName.getSelectedItem() ).getIndex();
        String str   = ( (IndexSelection) jComboBoxCharName.getSelectedItem() ).getName();
        return str;        
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    class KeySet
    {
        java.util.HashMap keys = new java.util.HashMap();
        java.lang.String  charStateName;
        
        public KeySet( String characterStateName )
        {
            if( characterStateName.startsWith("(" ) )
            {                
                // 2005.11.07 ??? Problem: Zeilen für legende zu lang
                System.out.println( "Map KeySet Don't add: " + characterStateName );
                charStateName = "TE";
                return;
            }
            charStateName = characterStateName;
        }
        public java.lang.String getCharacterStateName()
        {
            return charStateName;
        }
        public void add( String key, String value )
        {
            if( key == null )
            {
                System.out.println( "Map KeySet Don't add: null" );
                return; // 2005.11.07 ???
            }
/*            
            if( key.startsWith("(" ) )
            {                
                // 2005.11.07 ??? Problem: Zeilen f�r legende zu lang
                System.out.println( "Map KeySet Don't add: " + key );
                return;
            }
 */
            if( ! keys.containsKey( key ) )
            {
                // add new HashSet for this key
                keys.put( key, new java.util.HashSet() );
            }
            java.util.HashSet hs = (java.util.HashSet) keys.get( key );
            hs.add( value );
        }
        public int getCount( String key )
        {
            java.util.HashSet hs = (java.util.HashSet) keys.get( key );
            if( hs == null )
                return 0;
            else
                return hs.size();
        }
        public java.util.Set getKeys()
        {
            return keys.keySet();
        }
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    class IndexSelection
    {
        String  id;
        String  name;
        
        /**
         * @param id
         * @param name
         */        
        public IndexSelection( String id, String name )
        {
            this.id     = id;
            this.name   = name;            
        }
        /**
         * @return
         */        
        public String getIndex()
        {
            return id;
        }
        /**
         * @return
         */        
        public String getName()
        {
            return name;
        }
        /**
         * @return
         */        
        public java.lang.String toString()
        {
            return name;
        }
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    class DescriptionResultEntry
    {
        String  cs;
        String  name;
        
        /**
         * @param cs Character State
         *
         * @param name Character Name
         *
         */        
        public DescriptionResultEntry( String cs, String name )
        {
            this.cs     = cs;
            this.name   = CollectionTransectGisVisualisation.formatName( name );            
        }
        /**
         * @return Character State
         *
         */        
        public String getCS()
        {
            return cs;
        }
        /**
         * @return Character Name
         */        
        public String getName()
        {
            return name;
        }
        /**
         * @return Character Name
         */        
        public java.lang.String toString()
        {
            return name;
        }
    }
    //--------------------------------------------------------------------------
    /** Cut Author Name ... from input string
     *
     * Caloplaca sp. 1              --> Caloplaca sp. 1
     * Lecanora pseudistera Nyl.    --> Lecanora pseudistera
     * Peltula coriacea B�del, Henssen & Wessels    --> Peltula coriacea
     * Caloplaca volkii V. Wirth & Vezda            --> Caloplaca volkii
     * Letrouitia sp.1
     * @param str
     * @return
     */
    //--------------------------------------------------------------------------
    public static synchronized String formatName( String str )
    {
        java.util.StringTokenizer sb = new java.util.StringTokenizer( str, " \t", false );
        StringBuffer name = new StringBuffer();
        int count = 0;
        while( sb.hasMoreTokens() && count < 2 )
        {
            String token = sb.nextToken();
            if( count > 0 ) name.append( " " );
            if( ! token.endsWith( "." ) )
            {    
                count++;
            }
            name.append( token );            
        }
        return name.toString();
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private void jComboBoxCharNameActionPerformed(java.awt.event.ActionEvent evt)
    {
        // get Character state Selections List
        logger.debug( "CharName Action: " + evt );
        charNameSelection = ((IndexSelection) jComboBoxCharName.getSelectedItem()).getIndex();
        // The name are needed for matix-output
        charStateModel = setupCharacterStates( charNameSelection );
        if( ENABLE_CHAR_STATE_SELECTION )
        {
            jComboBoxCharState.setModel( new javax.swing.DefaultComboBoxModel( charStateModel ) );
        }
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private void jButtonShowImageActionPerformed(java.awt.event.ActionEvent evt)
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        StringBuffer cmd;        
        StringBuffer result = new StringBuffer();
	try
	{
            if( ENABLE_CHAR_STATE_SELECTION )
            {    
                charStateSelection = ((IndexSelection) jComboBoxCharState.getSelectedItem()).getIndex();
                if( charStateSelection.compareTo( "all" ) == 0 )
                {
                    getSelectedItems( charNameSelection, null );
                }
                else
                {
                    getSelectedItems( charNameSelection, charStateSelection );
                }
            }
            else
            {
                getSelectedItems( charNameSelection, null );
            }
            if( descriptionResults.size() == 0 )
            {
                DnOptionPane.showMessageDialog( null, "No results found in description database", "Error" , DnOptionPane.ERROR_MESSAGE );
                return;
            }
            // All CharacterStates und Names are now in "descriptionResults"
            // get coordinates, for each entry, from collection database
            
            DescriptionResultEntry e = (DescriptionResultEntry) descriptionResults.get( 0 );
            String entry        = e.getName();
            String charstate    = e.getCS();
            boolean charStateLoop = true;
            int i = 1;
            int charStateCount = 0;
            // Vector off KeySet's for each CharacterState 
            java.util.Vector keySetVector = new java.util.Vector();            
            while( charStateLoop )
            {   
                KeySet  ks = new KeySet( (java.lang.String) charStateHashMap.get( charstate ) );
                keySetVector.add( ks );
                // for all ChracterStates get the positions and retrieve the tansect
                charStateLoop = false;
                
                // use "CollectionGeography" to fetch locations 
                boolean USE_OBSERVATION_NAME_FROM_COLLECTION_SPECIMEN = false;  // if true, else use location from CollectionGeography
                
                if( debugFlag ) System.out.println( "--- " + charNameSelection + " " + ks.getCharacterStateName() +" ---" );
                
                if( USE_OBSERVATION_NAME_FROM_COLLECTION_SPECIMEN )
                {                   
                    cmd = new StringBuffer();
                    cmd.append( "SELECT DISTINCT "                          );	
                    cmd.append( "\"CollectionSpecimen\".\"DepositorsAccessionNumber\", "   );
                    cmd.append( "\"Identification\".\"TaxonomicName\" "     );
                    cmd.append( "FROM 	\"Identification\" , \"CollectionSpecimen\" "  );
                    cmd.append( "WHERE 	( \"Identification\".\"CollectionSpecimenID\" = \"CollectionSpecimen\".\"CollectionSpecimenID\" ) " );
                    if( descriptionResults.size() >= i )
                    {
                        cmd.append( "AND ( \"Identification\".\"TaxonomicName\"  LIKE   '" + entry + "%' " );
                    }
                }
                else
                {    
                    cmd = new StringBuffer();
                    cmd.append( "SELECT DISTINCT "                          );	
                    cmd.append( "\"CollectionGeography\".\"Location2\", "   );
                    cmd.append( "\"CollectionGeography\".\"Location1\", "   );
                    cmd.append( "\"Identification\".\"TaxonomicName\" "     );
                    cmd.append( "FROM 	\"Identification\" , \"CollectionGeography\" "  );
                    cmd.append( "WHERE 	( \"Identification\".\"CollectionSpecimenID\" = \"CollectionGeography\".\"CollectionSpecimenID\" ) " );
                    if( descriptionResults.size() >= i )
                    {
                        cmd.append( "AND ( \"Identification\".\"TaxonomicName\"  LIKE   '" + entry + "%' " );
                    }
                }
                
                for( ; i < descriptionResults.size(); i++ )
                {   
                    if( charstate.compareToIgnoreCase( ((DescriptionResultEntry) descriptionResults.get( i )).getCS() ) != 0 )
                    {
                        // This is the next characterState, build new query ....
                        charstate = ((DescriptionResultEntry) descriptionResults.get( i )).getCS();
                        entry = ((DescriptionResultEntry) descriptionResults.get( i )).getName();
                        charStateLoop = true;
                        break;
                    }    
                    entry = ((DescriptionResultEntry) descriptionResults.get( i )).getName();
                    cmd.append( "   OR \"Identification\".\"TaxonomicName\"  LIKE   '" + entry + "%' " );
                }
                cmd.append( " );" );

                Statement stmt = conCollection.createStatement();
                if( debugFlag ) System.out.println( cmd.toString() );
                ResultSet rs = stmt.executeQuery( cmd.toString() );
                if( debugFlag ) System.out.println( "result:" );
                charStateCount++;                
                while( rs.next() == true  )
                {
                    if( USE_OBSERVATION_NAME_FROM_COLLECTION_SPECIMEN )
                    {
                        if( debugFlag )
                        {
                            System.out.println( rs.getString( 1 ) + " \t" + formatName( rs.getString( 2 ) ) );
                        }
                        // ??? quick hack
                        // use the first 2 characters to determinate the oberservatory
                        ks.add( rs.getString( 1 ).substring( 0, 2 ), formatName( rs.getString( 2 ) ) );
                    }
                    else
                    {
                        // use location from CollectionGeography
                        if( debugFlag )
                        {
                            System.out.println( ct.getObservatory( rs.getString( 1 ), rs.getString( 2 ) ) + " \t" + formatName( rs.getString( 3 ) ) );
                        }
                        ks.add( ct.getObservatory( rs.getString( 1 ), rs.getString( 2 ) ), formatName( rs.getString( 3 ) ) );
                    }
                }
                stmt.close();
            }
            //---------------------------------------------
            // results: build output data for each transect
            //---------------------------------------------
            
            // ??? Quick hack for demo-map whisth R-Script
            // ??? OUTPUT:
            // ??? 1. Line Command line params
            // ???
            // ??? 2. Headline for legende
            // ??? .. Data 1. param Nuber off Location, ... data
            
            String seperator = ",";
            // list of characterStates
            
            // ??? Param-Error in R-Script
            // result.append( "\"Observatory\"," );
            result.append( "\"" + getCharacterName() + "\"\n" );
            result.append( "\"" + getCharacterName() + "\"" );
            result.append( ',' );
            for( int n = 0; n < keySetVector.size(); n++ )
            {
                if( n > 0 ) result.append( seperator );
                System.out.println( ((KeySet) keySetVector.get( n )).getCharacterStateName() );
                result.append( "\"" + ((KeySet) keySetVector.get( n )).getCharacterStateName() + "\"" );
            }
            // list of used Observatorys
            Object [] keys = ct.getObservatoryNames();
            java.util.Arrays.sort( keys );
            
            for( int trans = 0; trans < keys.length; trans++ )
            {
                String Observatory = (String) keys[ trans ];
                
                StringBuffer out = new StringBuffer( "\n" );
                // out.append( "Observatory: " + Observatory + " " );
                // cut first number
                out.append( Observatory.substring( 0, 2 ) );
                // while( out.length() < 20 ) out.append( " " );
                for( int charStateNum = 0; charStateNum < charStateCount; charStateNum++ )
                {   
                    KeySet ks = ( KeySet ) keySetVector.get( charStateNum );
                    // ??? quick hack
                    // int count = ks.getCount( Observatory );
                    int count = ks.getCount( Observatory.substring( 0, 2 ) );
                    out.append( seperator + count );
                }    
                result.append( out );
            }
            result.append( "\n" );
            System.out.println( result );
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        //----------------------------------------------------------------------
        // WEB-SERVICE CLIENT
        //
        // return Map-Filename        
        //----------------------------------------------------------------------
        String mapFileName = null;
        try
        {
            logger.info( "Build map" );
            Service service = new Service();
            Call    call    = ( Call ) service.createCall();

            call.setTargetEndpointAddress( new java.net.URL( WEB_SERVICE_ENDPOINT ) );
            call.setOperationName( "visualisation" );
            call.addParameter( "parameter", XMLType.XSD_STRING, ParameterMode.IN );
            call.setReturnType( XMLType.XSD_STRING );

            mapFileName = (String) call.invoke( new Object [] { result.toString() } );

            logger.info( "Get map: " + MAP_LOCATION + mapFileName + MAP_EXTENSION  );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getLocalizedMessage() );
        }   
        // System.out.println( buffer );
        // Show the new map with this locations
        try
        {
            imageViewer.showUrl( new java.net.URL( MAP_LOCATION + mapFileName + MAP_EXTENSION ) );
        }    
        catch( java.net.MalformedURLException ex )
        {
            logger.error( MAP_LOCATION + mapFileName + MAP_EXTENSION );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // 
    //--------------------------------------------------------------------------
    private java.util.Vector setupCharNames()
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        boolean newLineFlag = false;
        java.util.Vector values = new java.util.Vector();
        
	try
	{                
	    String cmd = "SELECT \"CID\", \"CharName\" FROM " + TABLE_CHAR + " WHERE \"Type\" = 'UM' OR \"Type\" = 'OM' ORDER BY \"CharName\";";
	    Statement stmt = conDescription.createStatement();
	    if( debugFlag ) System.out.println( cmd );
	    ResultSet rs = stmt.executeQuery( cmd );
	    if( debugFlag ) System.out.println( "result:" );

            String charName   = "";
            int    cid;
	    while( rs.next() == true  )
	    {
                newLineFlag = false;
		try
		{
                    values.add( new IndexSelection( rs.getString( 1 ),  rs.getString( 2 )) );
		}
		catch( Exception ex )
		{
		    System.out.println( ex.getMessage() );
		}
	    }
	    stmt.close();
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}                
        return values;
    }
    //--------------------------------------------------------------------------
    private java.util.Vector setupCharacterStates( String charNameId )
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        boolean newLineFlag = false;
        java.util.Vector values = new java.util.Vector();
                
	try
	{                
	    String cmd = "SELECT \"CS\", \"CharStateName\" FROM " + TABLE_CS + " WHERE \"CID\" = '" + charNameSelection + "' ORDER BY \"CharStateName\";";
	    Statement stmt = conDescription.createStatement();
	    if( debugFlag ) System.out.println( cmd );
	    ResultSet rs = stmt.executeQuery( cmd );
	    if( debugFlag ) System.out.println( "result:" );

            values.add( new IndexSelection( "all", "All character states" ) );
            charStateHashMap = new java.util.HashMap();
	    while( rs.next() == true  )
	    {
                newLineFlag = false;
		try
		{
                    values.add( new IndexSelection(rs.getString( 1 ),rs.getString( 2 )) );
                    charStateHashMap.put( rs.getString( 1 ),rs.getString( 2 ) );
		}
		catch( Exception ex )
		{
		    System.out.println( ex.getMessage() );
		}
	    }
	    stmt.close();
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}        
        
        return values;
    }
    //--------------------------------------------------------------------------
    /**
     * Build a list from description database for the selected items.
     * if( cs == null ) build a list for all posible character stats.
     *
     * @param cid
     * @param cs    Character State
     */    
    void getSelectedItems( String cid, String cs )
    {
        boolean debugFlag = false;
        
        StringBuffer cmd = new StringBuffer();        
        cmd.append( "SELECT DISTINCT "                      );
        cmd.append( TABLE_DESCR + ".\"CS\","           );
        // cmd.append( TABLE_DESCR + ".\"IID\","       );
        cmd.append( TABLE_DESCR + ".\"CID\","          );
        cmd.append( TABLE_CHAR + ".\"CharName\","      );
        cmd.append( TABLE_CS +".\"CharStateName\","    );
        cmd.append( TABLE_ITEM + ".\"ItemName\" "      );
        cmd.append( "FROM " +	TABLE_CS + ", " + TABLE_DESCR + ", " + TABLE_CHAR + ", " + TABLE_ITEM + " "  );
        cmd.append( "WHERE 	( "+ TABLE_DESCR + ".\"CID\" = '" + cid + "') "                         );
        if( cs != null )
        {
            cmd.append( "AND	( " + TABLE_CS +   ".\"CS\"   =  '" + cs + "') " );
        }
        cmd.append( "AND	( " + TABLE_CS +   ".\"CID\"  = " + TABLE_DESCR + ".\"CID\" ) " );
        cmd.append( "AND	( " + TABLE_CHAR + ".\"CID\"  = " + TABLE_DESCR + ".\"CID\" ) " );
        cmd.append( "AND	( " + TABLE_CS +   ".\"CS\"   = " + TABLE_DESCR + ".\"CS\"  ) " );
        cmd.append( "AND        ( " + TABLE_ITEM + ".\"IID\"  = " + TABLE_DESCR + ".\"IID\" ) " );
        cmd.append( "ORDER BY \"CS\";"                                                          );     
        
        descriptionResults = new java.util.Vector();
        
	try
	{                
	    Statement stmt = conDescription.createStatement();
	    if( debugFlag ) System.out.println( cmd.toString() );
	    ResultSet rs = stmt.executeQuery( cmd.toString() );
	    if( debugFlag ) System.out.println( "result:" );

            if( debugFlag ) System.out.println( "CID. " + cid + " ------------------------------------------------------------------" ); 
	    while( rs.next() == true  )
	    {
                if( debugFlag )
                {    
                    System.out.print( rs.getString( 1 ) + " \t" );
                    System.out.print( rs.getString( 2 ) + " \t" );
                    System.out.print( rs.getString( 3 ) + " \t" );
                    System.out.print( rs.getString( 4 ) + " \t" );
                    System.out.println( rs.getString( 5 ) );
                }
                descriptionResults.add( new  DescriptionResultEntry( rs.getString( 1 ), rs.getString( 5 ) ) );
	    }
	    stmt.close();
	}
	catch( java.sql.SQLException ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}        
    }
    //--------------------------------------------------------------------------
    /**
     * @param evt
     */    
    public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
    {
        logger.debug( "Exit: colse connection" );
        try
        {
            if( conCollection != null )
            {    
                logger.info( "ChecklistGisVisualisation: close database connection" );
                conCollection.close();
            }    
            if( conDescription != null )
            {    
                logger.info( "ChecklistGisVisualisation: close database connection" );
                conDescription.close();
            }    
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        System.exit( 0 );
    }    
    //--------------------------------------------------------------------------
    /** Only for testing
     * @param args the command line arguments
     * @throws Exception
     */
    public static void main( String args[] ) throws Exception
    {
        Connection conDesc = null;
        Connection conColl = null;
        try
        {
            Class.forName( "org.postgresql.Driver" );
            logger.info( "Load PostgreSQL Driver..." );
        }
        catch( java.lang.ClassNotFoundException ee )
        {
            logger.error( "ClassNotFoundException: " );
            logger.error( ee.getMessage() );
            System.err.print( "ClassNotFoundException: " );
            System.err.println( ee.getMessage() );
        }
        try
        {
            ConnectionInfo ciDesc = null;        
            ciDesc = net.metadiversity.diversity.navigator.connection.ConnectionsManager.getConnectionInfoByName( "UBT BIOTA Lichen Descriptions" );
            conDesc = ciDesc.getConnection();
            
            ConnectionInfo ciColl = null;        
            ciColl = net.metadiversity.diversity.navigator.connection.ConnectionsManager.getConnectionInfoByName( "UBT BIOTA Lichen Collections" );
            conColl = ciColl.getConnection();
            
            // conDesc = java.sql.DriverManager.getConnection( "jdbc:postgresql://hongkong:5432/ubt-description-94", "didi", "" );
            // conColl = java.sql.DriverManager.getConnection( "jdbc:postgresql://hongkong:5432/ubt-collection-94", "didi", "" );
        }
        catch( java.sql.SQLException ex )
        {
             logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        // only for testing
        final CollectionTransectGisVisualisation cgv = new CollectionTransectGisVisualisation( conDesc, conColl, "LIASlight" );               
        
        try
        {
            org.apache.log4j.BasicConfigurator.configure();
            logger.setLevel( Level.DEBUG );
            javax.swing.JFrame frame = new javax.swing.JFrame();
            frame.getContentPane().add( cgv, java.awt.BorderLayout.CENTER );
            frame.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
            frame.setSize( 500, 100 );
            frame.pack();
            frame.setVisible( true );

            frame.addWindowListener(new java.awt.event.WindowAdapter()
            {
                public void windowClosing(java.awt.event.WindowEvent evt)
                {
                    try
                    {
                        if( cgv.conDescription != null )
                        {
                            logger.info( "close connection" );
                            cgv.conDescription.close();
                        }    
                        if( cgv.conCollection != null )
                        {
                            logger.info( "close connection" );
                            cgv.conCollection.close();
                        }
                    }
                    catch( java.sql.SQLException ex )
                    {
                        logger.error( ex.getMessage(), ex.fillInStackTrace() );
                    }
                }
            });            
        }
        finally
        {
            
        }                
    }
}
