package net.metadiversity.diversity.navigator.gui;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.URL;
import java.util.List;
import java.util.ListIterator;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

import net.metadiversity.diversity.navigator.DiversityNavigatorProperties;


import org.apache.log4j.Logger;

/**
 * This frame shows the contents of a HTML file.
 *
 * @author  didi
 */
public class HtmlViewer extends JFrame implements Runnable
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3258131340938589489L;

    static Logger logger = Logger.getLogger( HtmlViewer.class.getName() );
    
    // get current directory and build the path for home-page
    //    String home = "file:/home/didi/bodsam/dnhelp/dn_help.html";  // A default value    
    
    private String home = "file:" + System.getProperty( "user.dir" ) + "/doc/index.html";
    
    static boolean mainFlag;   // true if main() usesd    
    static boolean exitWhenLastWindowClosed = false;
    /** Test HTML Viewer
     * @param args
     * @throws IOException
     */
    public static void main(String[] args) throws IOException 
    {
        logger.debug( "HtmlViewer: main()" );
	// End the program when there are no more open browser windows
	HtmlViewer.setMainFlag( true );
	HtmlViewer.setExitWhenLastWindowClosed( true );
        // Create a browser window
	HtmlViewer browser = new HtmlViewer( "Diversity Navigator Help", "index.html" );  
	browser.setSize( 600, 400 );
	browser.setVisible( true );   
	browser.displayPage( browser.getHome() );
        browser.start();
    }
    
    private Thread runner;
    
    public void start()
    {
        logger.debug( "HtmlViewer: start()" );
        runner = new Thread( this );
        runner.start();
    }
    /** When an object implementing interface <code>Runnable</code> is used
     * to create a thread, starting the thread causes the object's
     * <code>run</code> method to be called in that separately executing
     * thread.
     * <p>
     * The general contract of the method <code>run</code> is that it may
     * take any action whatsoever.
     *
     * @see     java.lang.Thread#run()
     *
     */
    public void run()
    {
        logger.debug( "HtmlViewer: run()" );
	// End the program when there are no more open browser windows
	HtmlViewer.setExitWhenLastWindowClosed( true );
        
/*
        try
        {
 */
            SwingUtilities.invokeLater( new Runnable()
            {
                public void run()
                {
                    try
                    {
                        // ??? contentsTxt.requestFocus();
                        // System.out.println( getHome() );
                        read( new URL( getHome() ) );            
                    }
                    catch( java.net.MalformedURLException ex )
                    {
                        logger.error( "Error reading URL" + ex.toString() );            
                    }
                   
                    catch( java.io.IOException ex )
                    {
                        logger.error( "Error reading URL" + ex.toString() );            
                    }                     
                }
            }); 
/*            
        }
        catch( IOException ex )
        {
            logger.error( "Error reading URL" + ex.toString() );            
        }
 */
    }
 
    /** Set the static property that controls the behavior of close()
     * @param b
     */
    public static void setExitWhenLastWindowClosed(boolean b) 
    {
        logger.debug( "HtmlViewer: setExitWhenLastWindowClosed( " + b + " )" );
	exitWhenLastWindowClosed = b;
    }
    public static void setMainFlag( boolean flag )
    {
        mainFlag = true;
    }
    /** These are accessor methods for the home property.
     * @param home
     */
    public void setHome( String home ) 
    { 
        logger.debug( "HtmlViewer: setHome( " + home + " )" );
        this.home = "file:" + System.getProperty( "user.dir" ) + "/doc/" + home;
        System.out.println( "HTML Viewer home : " + this.home );
        try
        {
            documentURL = new URL( this.home );
        }
        catch(  java.net.MalformedURLException ex )
        {   
            logger.error( "Bad URL: " + this.home );
        }
    }
    /**
     * @return
     */    
    public String getHome() 
    { 
        logger.debug( "HtmlViewer: getHome() --> " + home );
        return home; 
    }
    /** Like displayPage(URL), but takes a string instead
     * @param href
     */
    public void displayPage( String href ) 
    {
        logger.debug( "HtmlViewer: displayPage( " + href + " )" );
	try 
        {
	    logger.info( "Load URL: " + href );
	    displayURL( new URL( href ) );
	}
	catch( java.net.MalformedURLException ex ) 
        {
            message.setText( "Bad URL: " + href );
	    logger.error( "Bad URL: " + href );
	}
    }

   
    /** Toolbar for window. 
     */
    private javax.swing.JToolBar toolBar;
    /** Message. 
     */
    private javax.swing.JLabel message;
    /** Original URL (home). 
     */
    private URL documentURL;
    
    /** Current URL. 
     */
    private URL currentURL;
    
    /** Text area containing the HTML. 
     */
    private JEditorPane contentsTxt = new JEditorPane();
    
    /** <TT>JScrollPane</TT> for <TT>_contentsText</TT>. 
     */
    private JScrollPane contentsScrollPane;
    
    /** History of links. 
     */
    private List history = new LinkedList();
    
    /** Current index into <TT>_history</TT>. */
    private int historyIndex = -1;
    
    /**
     * @param title
     * @throws IOException
     */    
    public HtmlViewer( String title) throws IOException
    {
        this( title, null);
    }
    
    /**
     * @param title
     * @param url
     * @throws IOException
     */    
    public HtmlViewer( String title, String home ) throws IOException
    {
        super( title );
        logger.debug( "HtmlViewer: HtmlViewer( " + title + ", " +  home + " )" );
        createGUI();
        setVisible(true);   
        setHome( home );        
    }
    
    /** Display URL and store url as home.
     * @param url
     * @throws IOException
     */    
    public synchronized void read( URL url ) throws IOException
    {
        logger.debug( "HtmlViewer: read( " + url + " )" );
        if( url != null )
        {
            documentURL = url;            
            try
            {
                displayURL( url );
                history.add( url );
                historyIndex = 0;
            }
            finally
            {
            }
        }
    }
    
    /**
     * Return the URL being displayed.
     *
     * @return	URL being displayed.
     */
    public URL getURL()
    {
        logger.debug( "HtmlViewer: getUrl()" );
        return documentURL;
    }
    
    public void goBack()
    {
        if( historyIndex > 0 && historyIndex < history.size() )
        {
            displayURL( (URL)history.get( --historyIndex) );
        }
    }
    
    public void goForward()
    {
        if( historyIndex > -1 && historyIndex < history.size() - 1 )
        {
            displayURL( (URL)history.get( ++historyIndex) );
        }
    }
    
    public void refreshPage()
    {
        final Point pos = contentsScrollPane.getViewport().getViewPosition();
        displayURL( currentURL );
        contentsScrollPane.getViewport().setViewPosition( pos );
    }
    
    private void displayURL( URL url )
    {
        logger.debug( "HtmlViewer: displayUrl( " + url + " )" );
        if( url != null )
        {
            try
            {
                contentsTxt.setPage( url );
                currentURL = url;
                message.setText( url.toString() );
            }
            catch( Exception ex )
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                message.setText( "Can't read " + ex.getMessage() );
            }
        }
    }
    
    /**
     * Create user interface.
     */
    private void createGUI()
    {
        // Set DiversityNavigator Icon
        if( ! mainFlag )
        {    
            setIconImage( DiversityNavigatorProperties.getDiversityIcon().getImage() );
        }
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        Container contentPane = getContentPane();
        contentPane.setLayout( new BorderLayout() );
        message = new javax.swing.JLabel( " " );
        contentPane.add( createMainPanel(), BorderLayout.CENTER );
        contentPane.add( createToolBar(),   BorderLayout.NORTH  );
        contentPane.add( message,           BorderLayout.SOUTH  );
        pack();
        setSize( 600, 480 );
    }
    
    private javax.swing.JToolBar createToolBar()
    {
        toolBar = new javax.swing.JToolBar();
        toolBar.setBorder( BorderFactory.createEtchedBorder() );
        toolBar.setFloatable( false );

        java.lang.String iconPath = "/net/metadiversity/diversity/navigator/images/";

        //------------------
        // home
        //------------------
        javax.swing.JButton home = new javax.swing.JButton();
        home.setToolTipText( "load home page" );
        try
        {
            String icon = "Home24.gif";
            home.setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
        }
        catch( java.lang.NullPointerException ex )
        {
            logger.error( "icon for home not found" );
        }
        home.addActionListener( new HomeAction() );
        // toolBar.add( javax.swing.Box.createHorizontalStrut(10) ) ;
        toolBar.add( home ) ;
        //------------------
        // back
        //------------------
        javax.swing.JButton back = new javax.swing.JButton();
        back.setToolTipText( "" );
        try
        {
            String icon = "Back24.gif";
            back.setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
        }
        catch( java.lang.NullPointerException ex )
        {
            logger.error( "icon for back not found" );
        }
        back.addActionListener( new BackAction() );
        toolBar.add( back ) ;
        //------------------
        // Forward
        //------------------
        javax.swing.JButton forward = new javax.swing.JButton();
        forward.setToolTipText( "Forward" );
        try
        {
            String icon = "Forward24.gif";
            forward.setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
        }
        catch( java.lang.NullPointerException ex )
        {
            logger.error( "icon for forward not found" );
        }
        forward.addActionListener( new ForwardAction() );
        toolBar.add( forward ) ;
        //------------------
        // Refresh
        //------------------
        javax.swing.JButton refresh = new javax.swing.JButton();
        refresh.setToolTipText( "Refresh" );
        try
        {
            String icon = "Refresh24.gif";
            refresh.setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
        }
        catch( java.lang.NullPointerException ex )
        {
            logger.error( "icon for refresh not found" );
        }
        refresh.addActionListener( new RefreshAction() );
        toolBar.add( refresh ) ;
        
        //------------------
        // Close
        //------------------
        javax.swing.JButton close = new javax.swing.JButton();
        close.setToolTipText( "Close" );
        try
        {
            String icon = "Stop24.gif";
            close.setIcon( new javax.swing.ImageIcon( getClass().getResource( iconPath + icon ) ) );
        }
        catch( java.lang.NullPointerException ex )
        {
            logger.error( "icon for close not found" );
        }
        close.addActionListener( new CloseAction() );
        toolBar.add( close ) ;
        
        return toolBar;
    }
    
    /**
     * Create the main panel.
     */
    private JPanel createMainPanel()
    {
        contentsTxt.setEditable( false );
        contentsTxt.setContentType( "text/html" );
        final JPanel htmlPanel = new JPanel(new BorderLayout());
        contentsTxt.setBorder( BorderFactory.createEmptyBorder(0, 1, 0, 0) );
        contentsTxt.addHyperlinkListener( createHyperLinkListener() );
        contentsScrollPane = new JScrollPane(contentsTxt,
        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        htmlPanel.add( contentsScrollPane, BorderLayout.CENTER );
        
        return htmlPanel;
    }
    
    private HyperlinkListener createHyperLinkListener()
    {
        return new HyperlinkListener()
        {
            public void hyperlinkUpdate( HyperlinkEvent ev )
            {
                if( ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED )
                {
                    if( ev instanceof HTMLFrameHyperlinkEvent )
                    {
                        ((HTMLDocument)contentsTxt.getDocument()).processHTMLFrameHyperlinkEvent( (HTMLFrameHyperlinkEvent) ev );
                    }
                    else
                    {
                        try
                        {
                            final URL url = ev.getURL();
                            ListIterator it = history.listIterator( historyIndex + 1 );
                            while( it.hasNext() )
                            {
                                it.next();
                                it.remove();
                            }
                            history.add(url);
                            historyIndex = history.size() - 1;
                            contentsTxt.setPage(url);
                            currentURL = url;
                            message.setText( url.toString() );
                        }
                        catch( IOException ex )
                        {
                            logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        }
                    }
                }
            }
        };
    }
    
    
    private final class CloseAction extends javax.swing.AbstractAction
    {
        /**
         * Comment for <code>serialVersionUID</code>
         */
        private static final long serialVersionUID = 3257007644366942516L;
        public CloseAction()
        {
            super();
        }
        public void actionPerformed(ActionEvent evt)
        {
            logger.debug( "Action close" );
            dispose();
        }
    }
    
    private final class BackAction extends javax.swing.AbstractAction
    {
        /**
         * Comment for <code>serialVersionUID</code>
         */
        private static final long serialVersionUID = 3258132448955872049L;
        public BackAction()
        {
            super();
        }
        public void actionPerformed(ActionEvent evt)
        {
            logger.debug( "Action back" );
            goBack();
        }
    }
    
    private final class ForwardAction extends javax.swing.AbstractAction
    {
        /**
         * Comment for <code>serialVersionUID</code>
         */
        private static final long serialVersionUID = 3617576015723903280L;
        public ForwardAction()
        {
            super();
        }
        public void actionPerformed(ActionEvent evt)
        {
            logger.debug( "Action forward" );
            goForward();
        }
    }
    
    private final class RefreshAction extends javax.swing.AbstractAction
    {
        /**
         * Comment for <code>serialVersionUID</code>
         */
        private static final long serialVersionUID = 3906928963098784822L;
        public RefreshAction()
        {
            super();
        }
        public void actionPerformed(ActionEvent evt)
        {
            logger.debug( "Action refresh" );
            refreshPage();
        }
    }
    
    private final class HomeAction extends javax.swing.AbstractAction
    {
        /**
         * Comment for <code>serialVersionUID</code>
         */
        private static final long serialVersionUID = 3258688801873278514L;
        public HomeAction()
        {
            super();
        }
        public void actionPerformed(ActionEvent evt)
        {
            try
            {
                logger.debug( "Action home" );
                read( documentURL );
            }
            catch( IOException ex )
            {
                logger.error( "Error reading URL" + ex.toString() );            
            }
        }
    }
}
